@@ -31,6 +31,7 @@
 #include <fcntl.h>
 
 #include "scanutils.h"
+#include "tprintf.h"
 
 enum Flags {
   FL_SPLAT  = 0x01,   // Drop the value, do not assign
@@ -45,7 +46,8 @@ enum Ranks {
   RANK_INT  = 0,
   RANK_LONG = 1,
   RANK_LONGLONG = 2,
-  RANK_PTR      = 3 // Special value used for pointers
+  RANK_PTR      = INT_MAX // Special value used for pointers
+//  RANK_PTR      = 3 // Special value used for pointers
 };
 
 const enum Ranks kMinRank = RANK_CHAR;
@@ -183,7 +185,7 @@ double strtofloat(const char* s)
 {
   int minus = 0;
   int v = 0;
-  int d, c;
+  int d;
   int k = 1;
   int w = 0;
 
@@ -212,19 +214,49 @@ double strtofloat(const char* s)
   return minus ? -f : f;
 }
 
+static int tess_vfscanf(FILE* stream, const char *format, va_list ap);
+
+int tess_fscanf(FILE* stream, const char *format, ...)
+{
+  va_list ap;
+  int rv;
+
+  va_start(ap, format);
+  rv = tess_vfscanf(stream, format, ap);
+  va_end(ap);
+
+  return rv;
+}
+
+#ifdef EMBEDDED
 int fscanf(FILE* stream, const char *format, ...)
 {
   va_list ap;
   int rv;
 
   va_start(ap, format);
-  rv = vfscanf(stream, format, ap);
+  rv = tess_vfscanf(stream, format, ap);
   va_end(ap);
 
   return rv;
 }
 
-int vfscanf(FILE* stream, const char *format, va_list ap)
+int vfscanf(FILE* stream, const char *format, ...)
+{
+  va_list ap;
+  int rv;
+
+  va_start(ap, format);
+  rv = tess_vfscanf(stream, format, ap);
+  va_end(ap);
+
+  return rv;
+}
+#endif
+
+#ifndef _MSV_VER
+static
+int tess_vfscanf(FILE* stream, const char *format, va_list ap)
 {
   const char *p = format;
   char ch;
@@ -243,7 +275,7 @@ int vfscanf(FILE* stream, const char *format, va_list ap)
     ST_MATCH,         // Main state of %[ sequence
     ST_MATCH_RANGE,   // After - in a %[ sequence
   } state = ST_NORMAL;
-  char *oarg, *sarg = NULL;    // %s %c or %[ string argument
+  char *sarg = NULL;    // %s %c or %[ string argument
   enum Bail bail = BAIL_NONE;
   int sign;
   int converted = 0;    // Successful conversions
@@ -251,6 +283,7 @@ int vfscanf(FILE* stream, const char *format, va_list ap)
   int matchinv = 0;   // Is match map inverted?
   unsigned char range_start = 0;
   off_t start_off = ftell(stream);
+  double fval;
 
   // Skip leading spaces
   SkipSpace(stream);
@@ -412,7 +445,7 @@ int vfscanf(FILE* stream, const char *format, va_list ap)
                 break;
               }
 
-              double fval = streamtofloat(stream);
+              fval = streamtofloat(stream);
               switch(rank) {
                 case RANK_INT:
                   *va_arg(ap, float *) = static_cast<float>(fval);
@@ -536,8 +569,11 @@ int vfscanf(FILE* stream, const char *format, va_list ap)
 
   return converted;
 }
+#endif
 
+#ifdef EMBEDDED
 int creat(const char *pathname, mode_t mode)
 {
   return open(pathname, O_CREAT | O_TRUNC | O_WRONLY, mode);
 }
+#endif
