@@ -1,10 +1,10 @@
 /******************************************************************************
- **	Filename:    adaptive.c
- **	Purpose:     Adaptive matcher.
- **	Author:      Dan Johnson
- **	History:     Fri Mar  8 10:00:21 1991, DSJ, Created.
+ ** Filename:    adaptive.c
+ ** Purpose:     Adaptive matcher.
+ ** Author:      Dan Johnson
+ ** History:     Fri Mar  8 10:00:21 1991, DSJ, Created.
  **
- **	(c) Copyright Hewlett-Packard Company, 1988.
+ ** (c) Copyright Hewlett-Packard Company, 1988.
  ** Licensed under the Apache License, Version 2.0 (the "License");
  ** you may not use this file except in compliance with the License.
  ** You may obtain a copy of the License at
@@ -16,40 +16,40 @@
  ** limitations under the License.
  ******************************************************************************/
 
-/**----------------------------------------------------------------------------
+/*----------------------------------------------------------------------------
           Include Files and Type Defines
-----------------------------------------------------------------------------**/
+----------------------------------------------------------------------------*/
 #include "adaptive.h"
 #include "emalloc.h"
 #include "freelist.h"
 #include "globals.h"
+#include "classify.h"
 
 #ifdef __UNIX__
 #include <assert.h>
 #endif
 #include <stdio.h>
 
-/**----------------------------------------------------------------------------
+/*----------------------------------------------------------------------------
               Public Code
-----------------------------------------------------------------------------**/
+----------------------------------------------------------------------------*/
 /*---------------------------------------------------------------------------*/
-int AddAdaptedClass(ADAPT_TEMPLATES Templates,
-                    ADAPT_CLASS Class,
-                    CLASS_ID ClassId) {
-/*
- **	Parameters:
- **		Templates	set of templates to add new class to
- **		Class		new class to add to templates
- **		ClassId		class id to associate with new class
- **	Globals: none
- **	Operation: This routine adds a new adapted class to an existing
- **		set of adapted templates.
- **	Return: The class index of the new class.
- **	Exceptions: none
- **	History: Thu Mar 14 13:06:09 1991, DSJ, Created.
+/**
+ * This routine adds a new adapted class to an existing
+ * set of adapted templates.
+ *
+ * @param Templates set of templates to add new class to
+ * @param Class new class to add to templates
+ * @param ClassId class id to associate with new class
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Thu Mar 14 13:06:09 1991, DSJ, Created.
  */
+void AddAdaptedClass(ADAPT_TEMPLATES Templates,
+                     ADAPT_CLASS Class,
+                     CLASS_ID ClassId) {
   INT_CLASS IntClass;
-  CLASS_INDEX ClassIndex;
 
   assert (Templates != NULL);
   assert (Class != NULL);
@@ -58,29 +58,26 @@ int AddAdaptedClass(ADAPT_TEMPLATES Templates,
   assert (Class->NumPermConfigs == 0);
 
   IntClass = NewIntClass (1, 1);
-  ClassIndex = AddIntClass (Templates->Templates, ClassId, IntClass);
+  AddIntClass (Templates->Templates, ClassId, IntClass);
 
-  assert (Templates->Class[ClassIndex] == NULL);
-
-  Templates->Class[ClassIndex] = Class;
-
-  return (ClassIndex);
+  assert (Templates->Class[ClassId] == NULL);
+  Templates->Class[ClassId] = Class;
 
 }                                /* AddAdaptedClass */
 
 
 /*---------------------------------------------------------------------------*/
-void FreeTempConfig(TEMP_CONFIG Config) {
-/*
- **	Parameters:
- **		Config	config to be freed
- **	Globals: none
- **	Operation: This routine frees all memory consumed by a temporary
- **		configuration.
- **	Return: none
- **	Exceptions: none
- **	History: Thu Mar 14 13:34:23 1991, DSJ, Created.
+/**
+ * This routine frees all memory consumed by a temporary
+ * configuration.
+ *
+ * @param Config  config to be freed
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Thu Mar 14 13:34:23 1991, DSJ, Created.
  */
+void FreeTempConfig(TEMP_CONFIG Config) {
   assert (Config != NULL);
 
   destroy_nodes (Config->ContextsSeen, memfree);
@@ -99,16 +96,17 @@ void FreeTempProto(void *arg) {
 
 
 /*---------------------------------------------------------------------------*/
-ADAPT_CLASS NewAdaptedClass() {
-/*
- **	Parameters: none
- **	Globals: none
- **	Operation: This operation allocates and initializes a new adapted
- **		class data structure and returns a ptr to it.
- **	Return: Ptr to new class data structure.
- **	Exceptions: none
- **	History: Thu Mar 14 12:58:13 1991, DSJ, Created.
+/**
+ * This operation allocates and initializes a new adapted
+ * class data structure and returns a ptr to it.
+ *
+ * @return Ptr to new class data structure.
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Thu Mar 14 12:58:13 1991, DSJ, Created.
  */
+ADAPT_CLASS NewAdaptedClass() {
   ADAPT_CLASS Class;
   int i;
 
@@ -149,15 +147,19 @@ void free_adapted_class(ADAPT_CLASS adapt_class) {
 
 
 /*---------------------------------------------------------------------------*/
-ADAPT_TEMPLATES NewAdaptedTemplates() {
-/*
- **	Parameters: none
- **	Globals: none
- **	Operation:
- **	Return: none
- **	Exceptions: none
- **	History: Fri Mar  8 10:15:28 1991, DSJ, Created.
+namespace tesseract {
+/**
+ * Allocates memory for adapted tempates.
+ * each char in unicharset to the newly created templates
+ *
+ * @param PopulateFromUnicharset if true, add an empty class for
+ * @return Ptr to new adapted templates.
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Fri Mar  8 10:15:28 1991, DSJ, Created.
  */
+ADAPT_TEMPLATES Classify::NewAdaptedTemplates(bool InitFromUnicharset) {
   ADAPT_TEMPLATES Templates;
   int i;
 
@@ -165,14 +167,20 @@ ADAPT_TEMPLATES NewAdaptedTemplates() {
 
   Templates->Templates = NewIntTemplates ();
   Templates->NumPermClasses = 0;
+  Templates->NumNonEmptyClasses = 0;
 
-  for (i = 0; i < MAX_NUM_CLASSES; i++)
+  /* Insert an empty class for each unichar id in unicharset */
+  for (i = 0; i < MAX_NUM_CLASSES; i++) {
     Templates->Class[i] = NULL;
+    if (InitFromUnicharset && i < unicharset.size()) {
+      AddAdaptedClass(Templates, NewAdaptedClass(), i);
+    }
+  }
 
   return (Templates);
 
 }                                /* NewAdaptedTemplates */
-
+}  // namespace tesseract
 
 /*----------------------------------------------------------------------------*/
 void free_adapted_templates(ADAPT_TEMPLATES templates) {
@@ -188,17 +196,17 @@ void free_adapted_templates(ADAPT_TEMPLATES templates) {
 
 
 /*---------------------------------------------------------------------------*/
-TEMP_CONFIG NewTempConfig(int MaxProtoId) {
-/*
- **	Parameters:
- **		MaxProtoId	max id of any proto in new config
- **	Globals: none
- **	Operation: This routine allocates and returns a new temporary
- **		config.
- **	Return: Ptr to new temp config.
- **	Exceptions: none
- **	History: Thu Mar 14 13:28:21 1991, DSJ, Created.
+/**
+ * This routine allocates and returns a new temporary config.
+ *
+ * @param MaxProtoId  max id of any proto in new config
+ * @return Ptr to new temp config.
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Thu Mar 14 13:28:21 1991, DSJ, Created.
  */
+TEMP_CONFIG NewTempConfig(int MaxProtoId) {
   TEMP_CONFIG Config;
   int NumProtos = MaxProtoId + 1;
 
@@ -219,33 +227,35 @@ TEMP_CONFIG NewTempConfig(int MaxProtoId) {
 
 
 /*---------------------------------------------------------------------------*/
-TEMP_PROTO NewTempProto() {
-/*
- **	Parameters: none
- **	Globals: none
- **	Operation: This routine allocates and returns a new temporary proto.
- **	Return: Ptr to new temporary proto.
- **	Exceptions: none
- **	History: Thu Mar 14 13:31:31 1991, DSJ, Created.
+/**
+ * This routine allocates and returns a new temporary proto.
+ *
+ * @return Ptr to new temporary proto.
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Thu Mar 14 13:31:31 1991, DSJ, Created.
  */
+TEMP_PROTO NewTempProto() {
   return ((TEMP_PROTO)
     alloc_struct (sizeof (TEMP_PROTO_STRUCT), "TEMP_PROTO_STRUCT"));
 }                                /* NewTempProto */
 
 
 /*---------------------------------------------------------------------------*/
-void PrintAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
-/*
- **	Parameters:
- **		File		open text file to print Templates to
- **		Templates	adapted templates to print to File
- **	Globals: none
- **	Operation: This routine prints a summary of the adapted templates
- **		in Templates to File.
- **	Return: none
- **	Exceptions: none
- **	History: Wed Mar 20 13:35:29 1991, DSJ, Created.
+namespace tesseract {
+/**
+ * This routine prints a summary of the adapted templates
+ *  in Templates to File.
+ *
+ * @param File    open text file to print Templates to
+ * @param Templates adapted templates to print to File
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Wed Mar 20 13:35:29 1991, DSJ, Created.
  */
+void Classify::PrintAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
   int i;
   INT_CLASS IClass;
   ADAPT_CLASS AClass;
@@ -253,38 +263,41 @@ void PrintAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
   #ifndef SECURE_NAMES
   fprintf (File, "\n\nSUMMARY OF ADAPTED TEMPLATES:\n\n");
   fprintf (File, "Num classes = %d;  Num permanent classes = %d\n\n",
-    (Templates->Templates)->NumClasses, Templates->NumPermClasses);
-  fprintf (File, "Index Id  NC NPC  NP NPP\n");
+           Templates->NumNonEmptyClasses, Templates->NumPermClasses);
+  fprintf (File, "   Id  NC NPC  NP NPP\n");
   fprintf (File, "------------------------\n");
 
   for (i = 0; i < (Templates->Templates)->NumClasses; i++) {
     IClass = Templates->Templates->Class[i];
     AClass = Templates->Class[i];
-
-    fprintf (File, "%5d  %s %3d %3d %3d %3d\n",
-      i, unicharset.id_to_unichar(Templates->Templates->ClassIdFor[i]),
+    if (!IsEmptyAdaptedClass (AClass)) {
+      fprintf (File, "%5d  %s %3d %3d %3d %3d\n",
+        i, unicharset.id_to_unichar(i),
       IClass->NumConfigs, AClass->NumPermConfigs,
       IClass->NumProtos,
       IClass->NumProtos - count (AClass->TempProtos));
+    }
   }
   #endif
   fprintf (File, "\n");
 
 }                                /* PrintAdaptedTemplates */
+}  // namespace tesseract
 
 
 /*---------------------------------------------------------------------------*/
-ADAPT_CLASS ReadAdaptedClass(FILE *File) {
-/*
- **	Parameters:
- **		File	open file to read adapted class from
- **	Globals: none
- **	Operation: Read an adapted class description from File and return
- **		a ptr to the adapted class.
- **	Return: Ptr to new adapted class.
- **	Exceptions: none
- **	History: Tue Mar 19 14:11:01 1991, DSJ, Created.
+/**
+ * Read an adapted class description from File and return
+ * a ptr to the adapted class.
+ *
+ * @param File  open file to read adapted class from
+ * @return Ptr to new adapted class.
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Tue Mar 19 14:11:01 1991, DSJ, Created.
  */
+ADAPT_CLASS ReadAdaptedClass(FILE *File) {
   int NumTempProtos;
   int NumConfigs;
   int i;
@@ -328,17 +341,19 @@ ADAPT_CLASS ReadAdaptedClass(FILE *File) {
 
 
 /*---------------------------------------------------------------------------*/
-ADAPT_TEMPLATES ReadAdaptedTemplates(FILE *File) {
-/*
- **	Parameters:
- **		File	open text file to read adapted templates from
- **	Globals: none
- **	Operation: Read a set of adapted templates from File and return
- **		a ptr to the templates.
- **	Return: Ptr to adapted templates read from File.
- **	Exceptions: none
- **	History: Mon Mar 18 15:18:10 1991, DSJ, Created.
+namespace tesseract {
+/**
+ * Read a set of adapted templates from File and return
+ * a ptr to the templates.
+ *
+ * @param File  open text file to read adapted templates from
+ * @return Ptr to adapted templates read from File.
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Mon Mar 18 15:18:10 1991, DSJ, Created.
  */
+ADAPT_TEMPLATES Classify::ReadAdaptedTemplates(FILE *File) {
   int i;
   ADAPT_TEMPLATES Templates;
 
@@ -347,7 +362,7 @@ ADAPT_TEMPLATES ReadAdaptedTemplates(FILE *File) {
   fread ((char *) Templates, sizeof (ADAPT_TEMPLATES_STRUCT), 1, File);
 
   /* then read in the basic integer templates */
-  Templates->Templates = ReadIntTemplates (File, FALSE);
+  Templates->Templates = ReadIntTemplates (File);
 
   /* then read in the adaptive info for each class */
   for (i = 0; i < (Templates->Templates)->NumClasses; i++) {
@@ -356,25 +371,27 @@ ADAPT_TEMPLATES ReadAdaptedTemplates(FILE *File) {
   return (Templates);
 
 }                                /* ReadAdaptedTemplates */
+}  // namespace tesseract
 
 
 /*---------------------------------------------------------------------------*/
-PERM_CONFIG ReadPermConfig(FILE *File) {
-/*
- **	Parameters:
- **		File	open file to read permanent config from
- **	Globals: none
- **	Operation: Read a permanent configuration description from File
- **		and return a ptr to it.
- **	Return: Ptr to new permanent configuration description.
- **	Exceptions: none
- **	History: Tue Mar 19 14:25:26 1991, DSJ, Created.
+/**
+ * Read a permanent configuration description from File
+ * and return a ptr to it.
+ *
+ * @param File  open file to read permanent config from
+ * @return Ptr to new permanent configuration description.
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Tue Mar 19 14:25:26 1991, DSJ, Created.
  */
+PERM_CONFIG ReadPermConfig(FILE *File) {
   PERM_CONFIG Config;
   uinT8 NumAmbigs;
 
   fread ((char *) &NumAmbigs, sizeof (uinT8), 1, File);
-  Config = (PERM_CONFIG) Emalloc (sizeof (char) * (NumAmbigs + 1));
+  Config = (PERM_CONFIG) Emalloc (sizeof (UNICHAR_ID) * (NumAmbigs + 1));
   fread (Config, sizeof (UNICHAR_ID), NumAmbigs, File);
   Config[NumAmbigs] = -1;
 
@@ -384,17 +401,18 @@ PERM_CONFIG ReadPermConfig(FILE *File) {
 
 
 /*---------------------------------------------------------------------------*/
-TEMP_CONFIG ReadTempConfig(FILE *File) {
-/*
- **	Parameters:
- **		File	open file to read temporary config from
- **	Globals: none
- **	Operation:  Read a temporary configuration description from File
- **		and return a ptr to it.
- **	Return: Ptr to new temporary configuration description.
- **	Exceptions: none
- **	History: Tue Mar 19 14:29:59 1991, DSJ, Created.
+/**
+ * Read a temporary configuration description from File
+ * and return a ptr to it.
+ *
+ * @param File  open file to read temporary config from
+ * @return Ptr to new temporary configuration description.
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Tue Mar 19 14:29:59 1991, DSJ, Created.
  */
+TEMP_CONFIG ReadTempConfig(FILE *File) {
   TEMP_CONFIG Config;
 
   Config =
@@ -412,19 +430,19 @@ TEMP_CONFIG ReadTempConfig(FILE *File) {
 
 
 /*---------------------------------------------------------------------------*/
-void WriteAdaptedClass(FILE *File, ADAPT_CLASS Class, int NumConfigs) {
-/*
- **	Parameters:
- **		File		open file to write Class to
- **		Class		adapted class to write to File
- **		NumConfigs	number of configs in Class
- **	Globals: none
- **	Operation: This routine writes a binary representation of Class
- **		to File.
- **	Return: none
- **	Exceptions: none
- **	History: Tue Mar 19 13:33:51 1991, DSJ, Created.
+/**
+ * This routine writes a binary representation of Class
+ * to File.
+ *
+ * @param File    open file to write Class to
+ * @param Class   adapted class to write to File
+ * @param NumConfigs  number of configs in Class
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Tue Mar 19 13:33:51 1991, DSJ, Created.
  */
+void WriteAdaptedClass(FILE *File, ADAPT_CLASS Class, int NumConfigs) {
   int NumTempProtos;
   LIST TempProtos;
   int i;
@@ -459,17 +477,18 @@ void WriteAdaptedClass(FILE *File, ADAPT_CLASS Class, int NumConfigs) {
 
 
 /*---------------------------------------------------------------------------*/
-void WriteAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
-/*
- **	Parameters:
- **		File		open text file to write Templates to
- **		Templates	set of adapted templates to write to File
- **	Globals: none
- **	Operation: This routine saves Templates to File in a binary format.
- **	Return: none
- **	Exceptions: none
- **	History: Mon Mar 18 15:07:32 1991, DSJ, Created.
+namespace tesseract {
+/**
+ * This routine saves Templates to File in a binary format.
+ *
+ * @param File    open text file to write Templates to
+ * @param Templates set of adapted templates to write to File
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Mon Mar 18 15:07:32 1991, DSJ, Created.
  */
+void Classify::WriteAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
   int i;
 
   /* first write the high level adaptive template struct */
@@ -484,21 +503,22 @@ void WriteAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
       Templates->Templates->Class[i]->NumConfigs);
   }
 }                                /* WriteAdaptedTemplates */
+}  // namespace tesseract
 
 
 /*---------------------------------------------------------------------------*/
-void WritePermConfig(FILE *File, PERM_CONFIG Config) {
-/*
- **	Parameters:
- **		File	open file to write Config to
- **		Config	permanent config to write to File
- **	Globals: none
- **	Operation: This routine writes a binary representation of a
- **		permanent configuration to File.
- **	Return: none
- **	Exceptions: none
- **	History: Tue Mar 19 13:55:44 1991, DSJ, Created.
+/**
+ * This routine writes a binary representation of a
+ * permanent configuration to File.
+ * 
+ * @param File  open file to write Config to
+ * @param Config  permanent config to write to File
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Tue Mar 19 13:55:44 1991, DSJ, Created.
  */
+void WritePermConfig(FILE *File, PERM_CONFIG Config) {
   uinT8 NumAmbigs = 0;
 
   assert (Config != NULL);
@@ -512,18 +532,18 @@ void WritePermConfig(FILE *File, PERM_CONFIG Config) {
 
 
 /*---------------------------------------------------------------------------*/
-void WriteTempConfig(FILE *File, TEMP_CONFIG Config) {
-/*
- **	Parameters:
- **		File	open file to write Config to
- **		Config	temporary config to write to File
- **	Globals: none
- **	Operation: This routine writes a binary representation of a
- **		temporary configuration to File.
- **	Return: none
- **	Exceptions: none
- **	History: Tue Mar 19 14:00:28 1991, DSJ, Created.
+/**
+ * This routine writes a binary representation of a
+ * temporary configuration to File.
+ *
+ * @param File  open file to write Config to
+ * @param Config  temporary config to write to File
+ *
+ * @note Globals: none
+ * @note Exceptions: none
+ * @note History: Tue Mar 19 14:00:28 1991, DSJ, Created.
  */
+void WriteTempConfig(FILE *File, TEMP_CONFIG Config) {
   assert (Config != NULL);
                                  /* contexts not yet implemented */
   assert (Config->ContextsSeen == NULL);
