@@ -21,7 +21,6 @@
 ----------------------------------------------------------------------------**/
 #include "blobclass.h"
 #include "fxdefs.h"
-#include "variables.h"
 #include "extract.h"
 #include "efio.h"
 #include "callcpp.h"
@@ -34,8 +33,10 @@
 #define MAXFILENAME             80
 #define MAXMATCHES              10
 
-// define default font name to be used in training
-#define FONT_NAME       "UnknownFont"
+static const char kUnknownFontName[] = "UnknownFont";
+
+STRING_VAR(classify_font_name, kUnknownFontName,
+           "Default font name to be used in training");
 
 /**----------------------------------------------------------------------------
         Global Data Definitions and Declarations
@@ -43,34 +44,13 @@
 /* name of current image file being processed */
 extern char imagefile[];
 
-/* parameters used to control the training process */
-static const char *FontName = FONT_NAME;
-
 /**----------------------------------------------------------------------------
             Public Code
 ----------------------------------------------------------------------------**/
-/*---------------------------------------------------------------------------*/
-void InitBlobClassifierVars() {
-/*
- **      Parameters: none
- **      Globals:
- **              FontName        name of font being trained on
- **      Operation: Install blob classifier variables into the wiseowl
- **              variable system.
- **      Return: none
- **      Exceptions: none
- **      History: Fri Jan 19 16:13:33 1990, DSJ, Created.
- */
-  VALUE dummy;
-
-  string_variable (FontName, "FontName", FONT_NAME);
-
-}                                /* InitBlobClassifierVars */
-
 
 /*---------------------------------------------------------------------------*/
-void
-LearnBlob (TBLOB * Blob, TEXTROW * Row, char BlobText[])
+void LearnBlob(const STRING& filename,
+               TBLOB * Blob, TEXTROW * Row, const char* BlobText) {
 /*
  **      Parameters:
  **              Blob            blob whose micro-features are to be learned
@@ -79,7 +59,8 @@ LearnBlob (TBLOB * Blob, TEXTROW * Row, char BlobText[])
  **              TextLength      number of characters in blob
  **      Globals:
  **              imagefile       base filename of the page being learned
- **              FontName        name of font currently being trained on
+ **              classify_font_name
+ **                              name of font currently being trained on
  **      Operation:
  **              Extract micro-features from the specified blob and append
  **              them to the appropriate file.
@@ -87,13 +68,37 @@ LearnBlob (TBLOB * Blob, TEXTROW * Row, char BlobText[])
  **      Exceptions: none
  **      History: 7/28/89, DSJ, Created.
  */
-#define MAXFILENAME     80
-#define MAXCHARNAME     20
-#define MAXFONTNAME     20
 #define TRAIN_SUFFIX    ".tr"
-{
   static FILE *FeatureFile = NULL;
-  char Filename[MAXFILENAME];
+  STRING Filename(filename);
+
+  // If no fontname was set, try to extract it from the filename
+  STRING CurrFontName = classify_font_name;
+  if (CurrFontName == kUnknownFontName) {
+    // filename is expected to be of the form [lang].[fontname].exp[num]
+    // The [lang], [fontname] and [num] fields should not have '.' characters.
+    const char *basename = strrchr(filename.string(), '/');
+    const char *firstdot = strchr(basename ? basename : filename.string(), '.');
+    const char *lastdot  = strrchr(filename.string(), '.');
+    if (firstdot != lastdot && firstdot != NULL && lastdot != NULL) {
+      strncpy(&CurrFontName[0], firstdot + 1, lastdot - firstdot - 1);
+      CurrFontName[lastdot - firstdot - 1] = '\0';
+    }
+  }
+
+  // if a feature file is not yet open, open it
+  // the name of the file is the name of the image plus TRAIN_SUFFIX
+  if (FeatureFile == NULL) {
+    Filename += TRAIN_SUFFIX;
+    FeatureFile = Efopen(Filename.string(), "w");
+    cprintf("TRAINING ... Font name = %s\n", CurrFontName.string());
+  }
+
+  LearnBlob(FeatureFile, Blob, Row, BlobText, CurrFontName.string());
+}                                // LearnBlob
+
+void LearnBlob(FILE* FeatureFile, TBLOB* Blob, TEXTROW* Row,
+               const char* BlobText, const char* FontName) {
   CHAR_DESC CharDesc;
   LINE_STATS LineStats;
 
@@ -102,15 +107,9 @@ LearnBlob (TBLOB * Blob, TEXTROW * Row, char BlobText[])
   GetLineStatsFromRow(Row, &LineStats);
 
   CharDesc = ExtractBlobFeatures (Blob, &LineStats);
-
-  // if a feature file is not yet open, open it
-  // the name of the file is the name of the image plus TRAIN_SUFFIX
-  if (FeatureFile == NULL) {
-    strcpy(Filename, imagefile);
-    strcat(Filename, TRAIN_SUFFIX);
-    FeatureFile = Efopen (Filename, "w");
-
-    cprintf ("TRAINING ... Font name = %s.\n", FontName);
+  if (CharDesc == NULL) {
+    cprintf("LearnBLob: CharDesc was NULL. Aborting.\n");
+    return;
   }
 
   // label the features with a class name and font name
