@@ -19,28 +19,29 @@
           Include Files and Type Defines
 ----------------------------------------------------------------------------**/
 #include "normmatch.h"
-#include "clusttool.h"
-#include "normfeat.h"
-#include "debug.h"
-#include "const.h"
-#include "efio.h"
-#include "emalloc.h"
-#include "globals.h"
-#include "scanutils.h"
 
 #include <stdio.h>
 #include <math.h>
 
-/* define default filenames for training data */
-#define NORM_PROTO_FILE   "normproto"
+#include "classify.h"
+#include "clusttool.h"
+#include "const.h"
+#include "efio.h"
+#include "emalloc.h"
+#include "globals.h"
+#include "helpers.h"
+#include "normfeat.h"
+#include "scanutils.h"
+#include "unicharset.h"
+#include "varable.h"
 
-typedef struct
+struct NORM_PROTOS
 {
   int NumParams;
   PARAM_DESC *ParamDesc;
   LIST* Protos;
   int NumProtos;
-} NORM_PROTOS;
+};
 
 /**----------------------------------------------------------------------------
           Private Function Prototypes
@@ -55,24 +56,20 @@ void PrintNormMatch(FILE *File,
 NORM_PROTOS *ReadNormProtos(FILE *File);
 
 /**----------------------------------------------------------------------------
-        Global Data Definitions and Declarations
+        Variables
 ----------------------------------------------------------------------------**/
-/* global data structure to hold char normalization protos */
-static NORM_PROTOS *NormProtos;
-
-/* name of file containing char normalization protos */
-static const char *NormProtoFile = NORM_PROTO_FILE;
 
 /* control knobs used to control the normalization adjustment process */
-make_float_var (NormAdjMidpoint, 32.0, MakeNormAdjMidpoint,
-15, 16, SetNormAdjMidpoint, "Norm adjust midpoint ...")
-make_float_var (NormAdjCurl, 2.0, MakeNormAdjCurl,
-15, 17, SetNormAdjCurl, "Norm adjust curl ...")
+double_VAR(classify_norm_adj_midpoint, 32.0, "Norm adjust midpoint ...");
+double_VAR(classify_norm_adj_curl, 2.0, "Norm adjust curl ...");
+
 /**----------------------------------------------------------------------------
               Public Code
 ----------------------------------------------------------------------------**/
 /*---------------------------------------------------------------------------*/
-FLOAT32 ComputeNormMatch(CLASS_ID ClassId, FEATURE Feature, BOOL8 DebugMatch) {
+namespace tesseract {
+FLOAT32 Classify::ComputeNormMatch(CLASS_ID ClassId, FEATURE Feature,
+                                   BOOL8 DebugMatch) {
 /*
  **	Parameters:
  **		ClassId		id of class to match against
@@ -139,32 +136,7 @@ FLOAT32 ComputeNormMatch(CLASS_ID ClassId, FEATURE Feature, BOOL8 DebugMatch) {
   return (1.0 - NormEvidenceOf (BestMatch));
 }                                /* ComputeNormMatch */
 
-
-/*---------------------------------------------------------------------------*/
-void GetNormProtos() {
-/*
- **	Parameters: none
- **	Globals:
- **		NormProtoFile	name of file containing normalization protos
- **		NormProtos	global data structure to hold protos
- **	Operation: This routine reads in a set of character normalization
- **		protos from NormProtoFile and places them into NormProtos.
- **	Return: none
- **	Exceptions: none
- **	History: Wed Dec 19 16:24:25 1990, DSJ, Created.
- */
-  FILE *File;
-  STRING name;
-
-  name = language_data_path_prefix;
-  name += NormProtoFile;
-  File = Efopen (name.string(), "r");
-  NormProtos = ReadNormProtos (File);
-  fclose(File);
-
-}                                /* GetNormProtos */
-
-void FreeNormProtos() {
+void Classify::FreeNormProtos() {
   if (NormProtos != NULL) {
     for (int i = 0; i < NormProtos->NumProtos; i++)
       FreeProtoList(&NormProtos->Protos[i]);
@@ -174,28 +146,7 @@ void FreeNormProtos() {
     NormProtos = NULL;
   }
 }
-
-/*---------------------------------------------------------------------------*/
-void InitNormProtoVars() {
-/*
- **	Parameters: none
- **	Globals:
- **		NormProtoFile		filename for normalization protos
- **	Operation: Initialize the control variables for the normalization
- **				matcher.
- **	Return: none
- **	Exceptions: none
- **	History: Mon Nov  5 17:22:10 1990, DSJ, Created.
- */
-  VALUE dummy;
-
-  string_variable (NormProtoFile, "NormProtoFile", NORM_PROTO_FILE);
-
-  MakeNormAdjMidpoint();
-  MakeNormAdjCurl();
-
-}                                /* InitNormProtoVars */
-
+}  // namespace tesseract
 
 /**----------------------------------------------------------------------------
               Private Code
@@ -208,14 +159,14 @@ void InitNormProtoVars() {
  *       1 / (1 + (NormAdj / midpoint) ^ curl)
  **********************************************************************/
 FLOAT32 NormEvidenceOf(register FLOAT32 NormAdj) {
-  NormAdj /= NormAdjMidpoint;
+  NormAdj /= classify_norm_adj_midpoint;
 
-  if (NormAdjCurl == 3)
+  if (classify_norm_adj_curl == 3)
     NormAdj = NormAdj * NormAdj * NormAdj;
-  else if (NormAdjCurl == 2)
+  else if (classify_norm_adj_curl == 2)
     NormAdj = NormAdj * NormAdj;
   else
-    NormAdj = pow (NormAdj, NormAdjCurl);
+    NormAdj = pow(static_cast<double>(NormAdj), classify_norm_adj_curl);
   return (1.0 / (1.0 + NormAdj));
 }
 
@@ -242,8 +193,8 @@ void PrintNormMatch(FILE *File,
   FLOAT32 TotalMatch;
 
   for (i = 0, TotalMatch = 0.0; i < NumParams; i++) {
-    ParamMatch = (Feature->Params[i] - Mean (Proto, i)) /
-      StandardDeviation (Proto, i);
+    ParamMatch = (Feature->Params[i] - Mean(Proto, i)) /
+      StandardDeviation(Proto, i);
 
     fprintf (File, " %6.1f", ParamMatch);
 
@@ -257,7 +208,8 @@ void PrintNormMatch(FILE *File,
 
 
 /*---------------------------------------------------------------------------*/
-NORM_PROTOS *ReadNormProtos(FILE *File) {
+namespace tesseract {
+NORM_PROTOS *Classify::ReadNormProtos(FILE *File, inT64 end_offset) {
 /*
  **	Parameters:
  **		File	open text file to read normalization protos from
@@ -288,7 +240,8 @@ NORM_PROTOS *ReadNormProtos(FILE *File) {
   NormProtos->ParamDesc = ReadParamDesc (File, NormProtos->NumParams);
 
   /* read protos for each class into a separate list */
-  while (fscanf (File, "%s %d", unichar, &NumProtos) == 2) {
+  while ((end_offset < 0 || ftell(File) < end_offset) &&
+         fscanf(File, "%s %d", unichar, &NumProtos) == 2) {
     if (unicharset.contains_unichar(unichar)) {
       unichar_id = unicharset.unichar_to_id(unichar);
       Protos = NormProtos->Protos[unichar_id];
@@ -298,8 +251,8 @@ NORM_PROTOS *ReadNormProtos(FILE *File) {
       NormProtos->Protos[unichar_id] = Protos;
     } else
       cprintf("Error: unichar %s in normproto file is not in unichar set.\n");
+    SkipNewline(File);
   }
-
   return (NormProtos);
-
 }                                /* ReadNormProtos */
+}  // namespace tesseract
